/*$Id: acre_adjoint_gradient.c,v 1.1 2004/04/30 J. Malard Exp d3j540 $ */

#include "tao_solver.h"
#include "petsclog.h"
#include "petscblaslapack.h"
#include "acre.h"
#include "copyright.h"
#undef __FUNC__
#define __FUNC__ "acre_adjoint_gradient"
int acre_adjoint_gradient( TAO_APPLICATION reml, Vec iVec, Vec gVec,
	void * cptr)
{
	acre_cntxt *cntxt = (acre_cntxt *) cptr;
	int n, nq;
	int i, ierr, j;
	Vec aVec;
	Vec bVec;
	Vec fVec;
	Mat Qb;
	PetscScalar *ia, *da;
	PetscScalar zero=0.0;
	PetscScalar wg, wc;
	IS tiS;
	IS fiS;
	VecScatter iScatter;
	int inew;

	HELLO("acre_adjoint_gradient enter");

	/* Set up the problem */
	n = cntxt -> n;
	nq = cntxt -> nq;

	inew = acre_set_context(cntxt, iVec);

	ierr = VecGetArray( iVec, &ia );			CHKERRQ(ierr);
	ierr = VecGetArray( cntxt->sLDiag, &da );		CHKERRQ(ierr);
	ierr = VecSet( &zero, gVec);				CHKERRQ(ierr);

	/* Det C contribution */
	for ( i=0; i<nq; i++ )
	{
		if ( da[i]!=0 )
		{
			wc = 2*da[i]/ia[i];
			VecSetValue(gVec,i,wc,ADD_VALUES);
		}
	}

	ierr = VecRestoreArray( cntxt->sLDiag, &da );		CHKERRQ(ierr);
	ierr = VecRestoreArray( iVec, &ia );			CHKERRQ(ierr);

	/* Det R contribution */
	ierr = VecDuplicate ( cntxt->b, &aVec);			CHKERRQ(ierr);
	ierr = VecDuplicate ( cntxt->b, &bVec);			CHKERRQ(ierr);
	ierr = VecDuplicate ( cntxt->b, &fVec);			CHKERRQ(ierr);

	if ( cntxt->ofcn==ACRE_REML )
	{
		ierr = MatCreateMPIDense( PETSC_COMM_WORLD,
			PETSC_DECIDE, PETSC_DECIDE, cntxt->m,
			cntxt->rn, TAO_NULL, &Qb);		CHKERRQ(ierr);

		acre_fd_backward( cntxt, Qb );

		for ( i=0; i<nq; i++ )
		{
			wg = 0.0;
			acre_setup_scatter ( cntxt, i, aVec,
				bVec, &fiS, &tiS, &iScatter);
			for ( j=0; j<n; j++ )
			{
				ierr = acre_get_dense_matrix_column_vector (
					cntxt->B, aVec, j);	CHKERRQ(ierr);
				acre_fdl_times_vec ( cntxt, i,
					aVec, bVec, ACRE_NO_TRANSPOSE,
					&fiS, &tiS, &iScatter);
				ierr = KSPSetRhs(cntxt->sles,
					bVec);			CHKERRQ(ierr);
				ierr = KSPSetSolution(
					cntxt->sles, fVec);	CHKERRQ(ierr);
				ierr = KSPSolve( cntxt->sles );	CHKERRQ(ierr);
				ierr = acre_get_dense_matrix_column_vector(
					Qb, bVec, j);		CHKERRQ(ierr);
				ierr = VecDot( fVec,bVec, &wc);	CHKERRQ(ierr);
				wg -= wc;
			}
			VecSetValue( gVec, i, wg, ADD_VALUES);	CHKERRQ(ierr);
			acre_close_scatter ( cntxt, i, aVec,
				bVec, &fiS, &tiS, &iScatter);
		}

		/* Free work matrix */
		ierr = MatDestroy(Qb);				CHKERRQ(ierr);
	}

	/* Residual Term */
	for ( i=0; i<nq; i++ )
	{
		acre_setup_scatter ( cntxt, i, cntxt->v,
			bVec, &fiS, &tiS, &iScatter);
		acre_fdl_times_vec( cntxt, i, cntxt->u, bVec,
			ACRE_TRANSPOSE, &fiS, &tiS,
			&iScatter ) ;
		acre_close_scatter ( cntxt, i, cntxt->v, bVec,
			&fiS, &tiS, &iScatter);
		ierr = VecDot( cntxt->v, bVec, &wc );		CHKERRQ(ierr);
		wc *= -2;
		VecSetValue( gVec, i, wc, ADD_VALUES);		CHKERRQ(ierr);
	}

	/* Free remaining vectors */
	ierr = VecDestroy(bVec);				CHKERRQ(ierr);
	ierr = VecDestroy(fVec);				CHKERRQ(ierr);
	ierr = VecDestroy(aVec);				CHKERRQ(ierr);

	/* Sum all contributions */

	ierr = VecAssemblyBegin(gVec);				CHKERRQ(ierr);
	ierr = VecAssemblyEnd(gVec);				CHKERRQ(ierr);

	ierr = VecCopy(gVec,cntxt->gVec);			CHKERRQ(ierr);
	cntxt->bgSet = 1;

	HELLO("acre_adjoint_gradient exit");

	return 0;
}
