/*$Id: acre_likelihood.c,v 1.1 2004/04/30 Joel M. Malard Exp d3j540 $ */

#include <assert.h>
#include "tao_solver.h"
#include "acre.h"
#include "copyright.h"
#undef __FUNC__
#define __FUNC__ "acre_likelihood"

int acre_likelihood ( TAO_APPLICATION reml, Vec iVec, PetscScalar *fVal,
	void * cptr)
{
	/* Purpose:
	Compute the likelihood for the given input vector. 

	Arguments:
	reml (in)     : the non-linear solver context
	iVec (in)     : the current estimate for Theta
	fVal (out)    : the likelihood
	cptr (in/out) : the ACREE context

	Locals:

	*/
	acre_cntxt *cntxt = (acre_cntxt *) cptr;
	int n, nq;
	int i, ierr;
	int here;
	PetscScalar oval;
	PetscScalar *ia, *da;
	PetscScalar detC, detR, resNorm;
	int inew;

	HELLO("acre_likelihood enter");
	ierr = MPI_Comm_rank(MPI_COMM_WORLD,&here);
		assert(ierr==MPI_SUCCESS);

	/* Set up the problem */
	n = cntxt -> n;
	nq = cntxt -> nq;

	inew = acre_set_context(cntxt, iVec);
	if ( inew>0 && cntxt->bfSet!=0 )
	{
		*fVal = cntxt->fScal;
		return 0;
	}

	/* *fVal += 2*ln(|L|) */
	ierr = VecGetArray( iVec, &ia);				CHKERRQ(ierr);
	ierr = VecGetArray( cntxt->sLDiag, &da);		CHKERRQ(ierr);
	detC = 0.0;
	for ( i=0; i<nq; i++)
	{
		if ( da[i]<1 ) continue;
		oval = ia[i];
		oval = (oval<0.0) ? -oval : oval;
		detC += 2*da[i]*log((double) oval );
	}
	ierr = VecRestoreArray( cntxt->sLDiag, &da);		CHKERRQ(ierr);
	ierr = VecRestoreArray(iVec, &ia);			CHKERRQ(ierr);

	/* Compute the contribution of the determinant of R */
	detR = 0.0;
	if (cntxt->ofcn==ACRE_REML)
	{
		for(i=0;i<n;i++)
		{
			ierr = MatGetValues( cntxt->R, 1, &i, 
				1, &i, &oval);			CHKERRQ(ierr);
			oval = (oval<0.0) ? -oval : oval;
			if(oval<1.0 && HUGE*oval<2 )
			{ 
				if ( here==0 )
				{
					PetscPrintf(PETSC_COMM_WORLD,
					"Matrix R is numerically singular\n");
					VecView(iVec,PETSC_VIEWER_STDOUT_SELF);
				}
			}
			detR += 2*log((double) oval);
		}
	}

	/* Compute u^TCu = v^Tv where Lw = b and
	 *         v = (I-QQ^T) w = L^Tu
	 */
	ierr = VecDot(cntxt->v,cntxt->v,&resNorm);		CHKERRQ(ierr);
	
	/* Combine the values computed by multiple groups */
	*fVal = detC + detR + resNorm;
	*fVal /= cntxt->m;

	cntxt->fScal = *fVal;
	cntxt->bfSet = 1;

	/* play it safe */
	ierr = MPI_Bcast(fVal, 1, MPI_DOUBLE, 0,
		MPI_COMM_WORLD);
		assert(ierr==MPI_SUCCESS);

	HELLO("acre_likelihood exit");

	return 0;
}
