/*$Id: acre_matrix_combine.c,v 1.1 2004/04/30 Joel M. Malard Exp d3j540 $ */

#include "tao_solver.h"
#include "petscblaslapack.h"
#include "petsclog.h"
#include "acre.h"
#include "petsclog.h"
#include <assert.h>
#include <mpi.h>
#include <math.h>
#include "copyright.h"
#undef __FUNC__  
#define __FUNC__ "acre_matrix_combine"
int acre_matrix_combine(acre_cntxt *cntxt, Mat AL, Mat AG )
{
	int npes = cntxt->qc;
	int i, ierr, *disp, *count;
	int start, end, nrows;
	PetscScalar *lVals, *gVals;
	MPI_Comm comm = cntxt->rcomm;

	HELLO("acre_matrix_combine enter");

	ierr = MPI_Comm_size(comm, &npes);
		assert(ierr==MPI_SUCCESS);
	
	if ( npes==1 )
	{
		ierr = MatCopy(AL, AG,
			SAME_NONZERO_PATTERN);			CHKERRQ(ierr);
		return 0;
	}

	ierr = MatGetOwnershipRange(AL, &start, &end);		CHKERRQ(ierr);
	nrows = end-start;

	ierr = MatGetArray(AL, &lVals);
	ierr = MatGetArray(AG, &gVals);

	ierr = PetscMalloc(npes*sizeof(int),&count);		CHKERRQ(ierr);
	for ( i=0; i<npes; i++ )
		count[i] = nrows*(
			cntxt->rstart[i+1]-cntxt->rstart[i]);

	ierr = PetscMalloc(npes*sizeof(int),&disp);		CHKERRQ(ierr);
	for ( i=0; i<npes; i++ )
		disp[i] = (cntxt->rstart[i])*nrows;

	ierr = MPI_Allgatherv(lVals, (cntxt->rn)*nrows,
		MPI_DOUBLE, gVals, count, disp,
		MPI_DOUBLE, comm);
		assert(ierr==MPI_SUCCESS);

	ierr = MatAssemblyBegin(AG, MAT_FINAL_ASSEMBLY);	CHKERRQ(ierr);
	ierr = MatAssemblyEnd(AG, MAT_FINAL_ASSEMBLY);		CHKERRQ(ierr);

	ierr = MatRestoreArray(AL, &lVals);
	ierr = MatRestoreArray(AG, &gVals);

	PetscFree(count);
	PetscFree(disp);

	HELLO("acre_matrix_combine exit");

	return 0;
}
