/*$Id: acre_setup_scatter.c,v 1.1 2004/04/30 Joel M. Malard Exp d3j540 $ */

#include "tao_solver.h"
#include "petscblaslapack.h"
#include "petsclog.h"
#include "acre.h"
#include "petsclog.h"
#include <assert.h>
#include <mpi.h>
#include <math.h>
#include "copyright.h"
#undef __FUNC__
#define __FUNC__ "acre_setup_scatter"
int acre_setup_scatter( acre_cntxt *cntxt, int p, Vec u, Vec v,
  	IS *fiS, IS *tiS, VecScatter *iScatter)
{
	int ierr;
	int h, i;
	int start, end;
	int nnz;
	int *fromRow, *toRow;
	Vec w;
	PetscScalar *wvals;

	HELLO("acre_setup_scatter enter");

	ierr = VecDuplicate ( cntxt->b, &w );			CHKERRQ(ierr);
	ierr = MatGetColumnVector ( cntxt->sL, w, p );		CHKERRQ(ierr);
	ierr = VecGetOwnershipRange ( w, &start, &end );	CHKERRQ(ierr);
	ierr = VecGetArray ( w, &wvals );			CHKERRQ(ierr);

	for ( i=start, nnz=0; i<end; i++ ) 
		nnz += (wvals[i-start]!=0);

	fromRow = NULL;
	toRow = NULL;
	if ( nnz!=0 )
	{
		ierr = PetscMalloc(nnz*sizeof(int),&fromRow);	CHKERRQ(ierr);
		ierr = PetscMalloc(nnz*sizeof(int),&toRow);	CHKERRQ(ierr);
	}

	for ( i=start, h=0; i<end; i++ )
	{
		if(wvals[i-start]!=0)
		{
			fromRow[h] = (int) wvals[i-start]-1;
			toRow[h] = i;
			h += 1;
		}
	}

	ierr = VecRestoreArray ( w, &wvals );			CHKERRQ(ierr);
	ierr = VecDestroy( w );					CHKERRQ(ierr);

	ierr = ISCreateGeneral(PETSC_COMM_WORLD,nnz,
		fromRow,fiS);					CHKERRQ(ierr);
	ierr = ISCreateGeneral(PETSC_COMM_WORLD,nnz,
		toRow,tiS);					CHKERRQ(ierr);
	if ( nnz!=0 ) PetscFree(fromRow);
	if ( nnz!=0 ) PetscFree(toRow);

	ierr = VecScatterCreate( u, *fiS, v, *tiS, iScatter);	CHKERRQ(ierr);

	HELLO("acre_setup_scatter exit");

	return 0;
}
