/*$Id: acre_vector_combine.c,v 1.1 2004/04/30 Joel M. Malard Exp d3j540 $ */

#include "tao_solver.h"
#include "petscblaslapack.h"
#include "petsclog.h"
#include "acre.h"
#include "petsclog.h"
#include <assert.h>
#include <mpi.h>
#include <math.h>
#include "copyright.h"
#undef __FUNC__  
#define __FUNC__ "acre_vector_combine"
int acre_vector_combine(acre_cntxt *cntxt, Vec u )
{
	int start, end;
	int npes = cntxt->qc;
	int i, ierr, *rows;
	PetscScalar *uals, *vals;
	MPI_Comm comm = cntxt->rcomm;

	HELLO("acre_vector_combine enter");

	ierr = MPI_Comm_size(comm, &npes);
		assert(ierr==MPI_SUCCESS);
	
	if ( npes==1 ) return 0;

	ierr = VecGetOwnershipRange(u, &start, &end);		CHKERRQ(ierr);
	ierr = VecGetArray(u,&uals);				CHKERRQ(ierr);

	ierr = PetscMalloc((end-start)*sizeof(PetscScalar),
		&vals);						CHKERRQ(ierr);
	ierr = PetscMalloc((end-start)*sizeof(int),
		&rows);						CHKERRQ(ierr);

	ierr = MPI_Allreduce(uals,vals,end-start,
		MPI_DOUBLE,MPI_SUM,comm);
		assert(ierr==MPI_SUCCESS);
	ierr = VecRestoreArray(u,&uals);			CHKERRQ(ierr);
	for ( i=start; i<end; i++ ) rows[i-start] = i;
	ierr = VecSetValues(u,end-start,rows,vals,
		INSERT_VALUES);					CHKERRQ(ierr);
	ierr = VecAssemblyBegin(u);				CHKERRQ(ierr);
	ierr = VecAssemblyEnd(u);				CHKERRQ(ierr);

	PetscFree(rows);
	PetscFree(vals);

	HELLO("acre_vector_combine exit");

	return 0;
}
